@@ -6,16 +6,17 @@ use warnings;
 use URI;
 use YAML qw( DumpFile LoadFile );
 use HTTP::Request::Common;
+use URI;
 use LWP::UserAgent;
 use Log::Log4perl qw(:easy);
 use JSON qw( from_json );
 use MIME::Base64;
 use Moo;
 
-our $VERSION = "0.04";
+our $VERSION = "0.05";
 
-has client_id     => ( is => "ro" );
-has client_secret => ( is => "ro" );
+has client_id     => ( is => "rw" );
+has client_secret => ( is => "rw" );
 has local_uri      => ( 
   is      => "rw",
   default => "http://localhost:8082",
@@ -30,6 +31,7 @@ has scope       => ( is => "rw" );
 has token_uri   => ( is => "rw" );
 has redir_uri   => ( is => "rw" );
 has access_type => ( is => "rw" );
+has raise_error => ( is => "rw" );
 
 ###########################################
 sub redirect_uri {
@@ -59,12 +61,16 @@ sub full_login_uri {
     $full_login_uri->query_form (
       client_id     => $self->client_id(),
       response_type => "code",
-      redirect_uri  => $self->redirect_uri(),
+      (defined $self->redirect_uri() ?
+        ( redirect_uri  => $self->redirect_uri() ) :
+        ()
+      ),
       scope         => $self->scope(),
       ($self->access_type() ?
           (access_type => $self->access_type()) : ()),
     );
 
+    DEBUG "full login uri: $full_login_uri";
     return $full_login_uri;
 }
 
@@ -222,8 +228,9 @@ sub tokens_get {
     my $resp = $ua->request($req);
 
     if( $resp->is_success() ) {
-        my $data = 
-        from_json( $resp->content() );
+        my $json = $resp->content();
+        DEBUG "Received: [$json]", 
+        my $data = from_json( $json );
 
         return ( $data->{ access_token }, 
             $data->{ refresh_token },
@@ -254,6 +261,52 @@ sub tokens_collect {
     $self->cache_write( $cache );
 }
 
+###########################################
+sub http_get {
+###########################################
+    my( $self, $url, $query ) = @_;
+
+    my $ua = LWP::UserAgent->new();
+
+    my $uri = URI->new( $url );
+    $uri->query_form( @$query ) if defined $query;
+
+    my $resp = $ua->get( $uri, 
+        $self->authorization_headers, @$query );
+
+    if( $resp->is_error ) {
+        if( $self->raise_error ) {
+            die $resp->message;
+        }
+        return undef;
+    }
+
+    return $resp->decoded_content;
+}
+
+###########################################
+sub client_init_conf_check {
+###########################################
+    my( $self, $url ) = @_;
+
+    my $conf = { };
+    if( -f $self->cache_file_path ) {
+        $conf = $self->cache_read();
+    }
+
+    if( !exists $conf->{ client_id } or
+        !exists $conf->{ client_secret } ) {
+        die "You need to register your application on " .
+          "$url and add the client_id and " .
+          "client_secret entries to " . $self->cache_file_path . "\n";
+    }
+    
+    $self->client_id( $conf->{ client_id } );
+    $self->client_secret( $conf->{ client_secret } );
+
+    return 1;
+}
+
 1;
 
 __END__
@@ -278,15 +331,33 @@ web services requiring OAuth access tokens.
 =head1 WARNING: LIMITED ALPHA RELEASE
 
 While C<OAuth::Cmdline> has been envisioned to work with 
-various OAuth-controlled web services, it currently only works with the
-Google Drive API and Spotify. 
+various OAuth-controlled web services, it is currently tested with the
+following services, shown below with their subclasses:
+
+=over
+
+=item B<OAuth::Cmdline::GoogleDrive>
+- Google Drive
+
+=item B<OAuth::Cmdline::Spotify>
+- Spotify
+
+=item B<OAuth::CmdLine::MicrosoftOnline>
+- Azure AD and other OAuth2-authenticated services that use the Microsoft
+Online common authentication endpoint (tested with Azure AD via the Graph
+API)
+
+=item B<OAuth::Cmdline::Automatic>
+- Automatic.com car plugin
+
+=item B<OAuth::Cmdline::Youtube>
+- Youtube viewer reports
+
+=back
+
 But stay tuned, I'll refactor the site-specific parts of the
 code soon, so that it'll work with Evernote, Tumblr and others as 
 well. Hey, or send me a pull request if you want to beat me to it! :)
-So far the following subclasses are available:
-
-    OAuth::Cmdline::GoogleDrive
-    OAuth::Cmdline::Spotify
 
 If you want to use this module for a different service, go ahead and try
 it, it might just as well work. In this case, specify the C<site> parameter,
@@ -312,8 +383,8 @@ Then, run the following script (the example uses the Spotify web service)
     my $oauth = OAuth::Cmdline::GoogleDrive->new(
         client_id     => "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
         client_secret => "YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY",
-        login_uri     => "https://accounts.spotify.com/authorize",
-        token_uri     => "https://accounts.spotify.com/api/token",
+	login_uri     => "https://accounts.google.com/o/oauth2/auth",
+	token_uri     => "https://accounts.google.com/o/oauth2/token",
         scope         => "user-read-private",
     );
     
@@ -353,7 +424,8 @@ include it in the request to the web service.
 =item C<new()>
 
 Instantiate a new OAuth::Cmdline::XXX object. XXX stands for the specific
-site's implementation, and can be "GoogleDrive" or "Spotify" or similar.
+site's implementation, and can be "GoogleDrive" or one of the other
+subclasses listed above.
 
 =item C<authorization_headers()>
 
